#include "opendefs.h"
#include "openmqtt.h"
#include "mrandom.h"
#include "openqueue.h"
#include "openserial.h"
#include "packetfunctions.h"
#include "scheduler.h"
#include "IEEE802154E.h"
#include "idmanager.h"
#include "time.h"
#include "stdlib.h"

//=========================== variables =======================================

mrandom_vars_t mrandom_vars_send;
uint8_t mrandom_dst_addr[]   = {
   0xbb, 0xbb, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01
};
MQTTSNString Topic;
MQTTSNString Msg;
MQTTSN_topicid topic_publish;
bool enviado = FALSE;
uint16_t contador = 0;

//=========================== prototypes ======================================

void mrandom_timer_cb(opentimers_id_t id);
void mrandom_task_cb(void);
void mrandom_timer_init(mrandom_vars_t *send, uint16_t period_aux);
void mrandom_register_socket(mrandom_vars_t *mrandom, bool send_receive);

//=========================== public ==========================================

void mrandom_init(void) {
   mrandom_register_socket(&mrandom_vars_send, TRUE);
   mrandom_timer_init(&mrandom_vars_send, MRANDOM_PERIOD_MS);
}

void mrandom_sendDone(OpenQueueEntry_t* msg, owerror_t error) {
   openqueue_freePacketBuffer(msg);
}

void mrandom_receive(OpenQueueEntry_t* msg) {
   openqueue_freePacketBuffer(msg);
}

//=========================== private =========================================

/**
\note timer fired, but we don't want to execute task in ISR mode instead, push
   task to scheduler with CoAP priority, and let scheduler take care of it.
*/
void mrandom_timer_cb(opentimers_id_t id){

   scheduler_push_task(mrandom_task_cb,TASKPRIO_COAP);
}

void mrandom_task_cb(void) {

      Topic.cstring = "Pruebas";
      Msg.cstring = "MQTTSNClient";

      if(ieee154e_isSynch() == FALSE)
	    return;

      if (idmanager_getIsDAGroot()){
	    opentimers_destroy(mrandom_vars_send.timerId);
	    return;
      }

      if(!openmqtt_isConnected()){
	    openmqtt_connect(mrandom_dst_addr,1,&Topic, &Msg);
	    return;
      }

      uint8_t low_num = 0, hi_num = 101, rc = 0;
      srand(time(NULL));
      uint16_t random_number= (uint16_t)(rand() % (hi_num - low_num)) + low_num;

      uint8_t payload[2];
      payload[1] = (uint8_t)((random_number & 0xff00)>>8);
      payload[0] = (uint8_t)(random_number & 0x00ff);

      topic_publish.type = MQTTSN_TOPIC_TYPE_NORMAL;
      topic_publish.data.long_.name = Topic.cstring;
      topic_publish.data.long_.len = strlen(Topic.cstring);

      rc = openmqtt_publish(&topic_publish, &payload[0], sizeof(payload));
      if(rc == SUCCESS){
	    printf("Exito!!! El valor enviado es %d\n", random_number);
      } else {
	    openmqtt_disconnect();
      }
}

void mrandom_register_socket(mrandom_vars_t *mrandom, bool send_receive){
   memset(mrandom,0,sizeof(mrandom_vars_t));
   mrandom->desc.port              = WKP_UDP_RANDOM_SEND;
   mrandom->desc.callbackSendDone  = NULL;
   mrandom->desc.callbackReceive   = NULL;
   openudp_register(&mrandom->desc);
}

void mrandom_timer_init(mrandom_vars_t *send, uint16_t period_aux){
  send->period = period_aux;
  // start periodic timer
  send->timerId = opentimers_create();
  opentimers_scheduleIn(
      send->timerId,
      period_aux,
      TIME_MS,
      TIMER_PERIODIC,
      mrandom_timer_cb
  );
}
