#include "read_and_forward.h"
#include "MQTTSNConnect.h"
#include "MQTTSNPacket.h"
#include "MQTTSNPublish.h"
#include "paho.mqtt.c/src/MQTTClient.h"



int main(void)
{
   struct sockaddr_in6 si_me, si_other;
   int s, i, slen = sizeof(si_other) ,
	 recv_len = 0, datalen = 0, len = 0,
	 rc = 0, QoS = 0, connack_rc = 0, payloadlen = 0, duration = 0;
   bool isConnected = 0;
   unsigned short packetid = 0;
   MQTTSNString Topic;
   char Topic_char[BUFLEN];
   MQTTSNString Msg;
   MQTTSNPacket_connectData data;
   unsigned char buf[BUFLEN], willRetain[BUFLEN], *payload, dup[BUFLEN];
   char straddr[INET6_ADDRSTRLEN];
   s = socket_init6(&si_me);
   MQTTClient client;
   MQTTClient_connectOptions conn_opts = MQTTClient_connectOptions_initializer;
   MQTTSN_topicid topic_cid;



   //keep listening for data
   while(1){
      printf("Waiting for data...\n");
      fflush(stdout);
      //try to receive some data, this is a blocking call
      socket_send_and_receive(&s,buf,&si_other,&slen,&recv_len,true);

      //print details of the client/peer and the data received
      printf("Received packet from %s:%d\n",
	     inet_ntop(AF_INET6, &si_other.sin6_addr, straddr, sizeof(straddr)),
	     ntohs(si_other.sin6_port));

      len = MQTTSNPacket_decode(buf, recv_len, &datalen);
      rc=buf[len];

      switch(rc){
      case MQTTSN_CONNECT:
	    printf("Connect llego\n");
	    len = MQTTSNDeserialize_connect(&data,buf,recv_len);
	    len = MQTTSNSerialize_willtopicreq(buf,BUFLEN);
	    socket_send_and_receive(&s,buf,&si_other,&slen,&len,false);
	    break;
      case MQTTSN_WILLTOPIC:
	    printf("WillTopic llego\n");
	    len = MQTTSNDeserialize_willtopic(&QoS, willRetain, &Topic, buf, recv_len);
	    strcpy(Topic_char, Topic.lenstring.data);
	    len = MQTTSNSerialize_willmsgreq(buf,BUFLEN);
	    socket_send_and_receive(&s,buf,&si_other,&slen,&len,false);
	    break;
      case MQTTSN_WILLMSG:
	    printf("WillMsg llego\n");
	    len = MQTTSNDeserialize_willmsg(&Msg, buf, recv_len);
	    printf("Realiza conexion con Mosquito\n");
	    connack_rc = mqtt_connect(&client, &conn_opts, data.clientID.lenstring.data);
	    if(connack_rc == 0)
		  isConnected = 1;
	    else
		  isConnected = 0;
	    len = MQTTSNSerialize_connack(buf, BUFLEN, connack_rc);
	    socket_send_and_receive(&s,buf,&si_other,&slen,&len,false);
	    break;
      case MQTTSN_PUBLISH:
	    printf("Publish llego\n");
	    if(!isConnected){
		  printf("No esta conectado\n");
		  mqtt_disconect(&client);
		  len = MQTTSNSerialize_disconnect(buf,BUFLEN,0);
		  socket_send_and_receive(&s,buf,&si_other,&slen,&len,false);
		  break;
	    }

	    len = MQTTSNDeserialize_publish(dup,&QoS,willRetain,&packetid,
					    &topic_cid,&payload,&payloadlen,
					    buf,recv_len);

	    rc = mqtt_publish(&client,Topic_char,payload,&payloadlen);
	    if(rc != 0){
		  printf("Es necesario desconectar y volver a conectar\n");
		  mqtt_disconect(&client);
		  len = MQTTSNSerialize_disconnect(buf,BUFLEN,0);
		  socket_send_and_receive(&s,buf,&si_other,&slen,&len,false);
		  break;
	    }
	    len = MQTTSNSerialize_puback(buf,BUFLEN, topic_cid.data.id, packetid, rc);
	    socket_send_and_receive(&s,buf,&si_other,&slen,&len,false);
	    break;
      case MQTTSN_DISCONNECT:
	    printf("Disconnect llego\n");
	    isConnected = 0;
	    len = MQTTSNDeserialize_disconnect(&duration, buf,recv_len);
	    mqtt_disconect(&client);
	    len = MQTTSNSerialize_disconnect(buf,BUFLEN,0);
	    socket_send_and_receive(&s,buf,&si_other,&slen,&len,false);
	    break;
      default:
	    printf("El codigo que llega es %d\n", rc);
      }
      for(i=0;i<BUFLEN;i++)
	    buf[i]='\0';

   }
   close(s);
   return 0;
}

int mqtt_disconect(MQTTClient *client){
      int rc = 0;
      rc = MQTTClient_disconnect(*client, 10000);
      MQTTClient_destroy(client);
      return rc;
}

int mqtt_connect(MQTTClient *client,
		 MQTTClient_connectOptions *conn_opts,
		 char * clientID){

      int rc = 0;
      MQTTClient_create(client, ADDRESS, CLIENTID,
			MQTTCLIENT_PERSISTENCE_NONE, NULL);
      conn_opts->keepAliveInterval = 20;
      conn_opts->cleansession = 1;

      if ((rc = MQTTClient_connect(*client, conn_opts)) != MQTTCLIENT_SUCCESS)
      {
	    printf("Erro al conectar, codigo de retorno %d\n", rc);
	    return rc;
      }
      return rc;
}

int mqtt_publish(MQTTClient *client,
		 char *Topic,
		 unsigned char *payload,
		 int *payloadlen){
      int rc = 0;
      MQTTClient_message pubmsg = MQTTClient_message_initializer;
      MQTTClient_deliveryToken token;

      pubmsg.payload = payload;
      pubmsg.payloadlen = *payloadlen;
      pubmsg.qos = QOS;
      pubmsg.retained = 0;
      MQTTClient_publishMessage(*client, Topic, &pubmsg, &token);
      rc = MQTTClient_waitForCompletion(*client, token, TIMEOUT);
      return rc;
}

int socket_init6(struct sockaddr_in6 *si_me){
   int s = 0;
   //create a UDP socket
   if ((s=socket(PF_INET6, SOCK_DGRAM, 0)) == -1){
      die("socket");
   }
   // zero out the structure
   memset((char *)si_me, 0, sizeof(si_me));
   si_me->sin6_family = AF_INET6;
   si_me->sin6_port = htons(PORT);
   si_me->sin6_addr = in6addr_any;
   //bind socket to port
   if( bind(s,(struct sockaddr *)si_me, sizeof(*si_me) ) == -1){
      die("bind");
   }
   return s;
}

void socket_send_and_receive(int *s,
			     char *buf,
			     struct sockaddr_in6 *si_other,
			     int *slen,
			     int *recv_len,
			     bool send_receive){
   if(send_receive){
     if ((*recv_len = recvfrom(*s, buf, BUFLEN, 0, (struct sockaddr *)si_other, slen)) == -1){
	die("recvfrom()");
     }
   } else {
     if (sendto(*s, buf, *recv_len, 0, (struct sockaddr*)si_other, *slen) == -1){
	die("sendto()");
     }
   }
}


void die(char *s){
   perror(s);
   exit(1);
}
