#include "opendefs.h"
#include "openmqtt.h"
#include "openqueue.h"
#include "openserial.h"
#include "packetfunctions.h"
#include "scheduler.h"
#include "IEEE802154E.h"
#include "idmanager.h"
#include <string.h>

//=========================== defines =========================================


//=========================== variables =======================================

openmqtt_vars_t openmqtt_vars;
bool isConnected = FALSE;
bool puback_recv = FALSE;
uint8_t openmqtt_dst_addr_connect[16];
MQTTSNString Topic;
MQTTSNString Msg;
MQTTSN_topicid topic_snd;

//=========================== prototype =======================================


//=========================== public ==========================================

//===== from stack

/**
\brief Initialize this module.
*/
void openmqtt_init(void) {

      memset(&openmqtt_vars,0,sizeof(openmqtt_vars_t));
      
      // register at UDP stack
      openmqtt_vars.desc.port              = WKP_UDP_MQTT;
      openmqtt_vars.desc.callbackReceive   = &openmqtt_receive;
      openmqtt_vars.desc.callbackSendDone  = &openmqtt_sendDone;
      openudp_register(&openmqtt_vars.desc);
}


void openmqtt_receive(OpenQueueEntry_t* msg) {
      
      uint8_t rc = FAILURE;
      uint8_t sendbuf[MAX_PACKET_SIZE];
      uint8_t rcvbuff[MAX_PACKET_SIZE];
      int datalen = 0, len = 0, connack_rc = 255;
 
      memcpy(rcvbuff,&msg->payload[0],msg->length);
      len = MQTTSNPacket_decode(rcvbuff, msg->length, &datalen);
      rc=rcvbuff[len];
      switch(rc){
      case MQTTSN_CONNACK:
	    if (MQTTSNDeserialize_connack(&connack_rc, rcvbuff, MAX_PACKET_SIZE) == 1){
		  if(SUCCESS == connack_rc){
			isConnected = TRUE;
			puback_recv = TRUE;
		  }
		  return;
	    }
	    break;
      case MQTTSN_WILLTOPICREQ:
	    if (MQTTSNDeserialize_willtopicreq(rcvbuff, MAX_PACKET_SIZE) == 1){
		  len = MQTTSNSerialize_willtopic(sendbuf, MAX_PACKET_SIZE, 0, 0, Topic);
		  openmqtt_sendPackage(&sendbuf[0], len);
		  return;
	    }
	    break;
      case MQTTSN_WILLMSGREQ:
	    if (MQTTSNDeserialize_willmsgreq(rcvbuff, MAX_PACKET_SIZE) == 1){
		  len = MQTTSNSerialize_willmsg(sendbuf, MAX_PACKET_SIZE, Msg);
		  openmqtt_sendPackage(&sendbuf[0], len);
		  return;
	    }
	    break;
      case MQTTSN_PUBACK:
	    puback_recv = TRUE;
	    break;
      case MQTTSN_DISCONNECT:
	    isConnected = FALSE;
	    return;
	    break;
      default:
	    break;
      }
      openqueue_freePacketBuffer(msg);
      return;
}

void openmqtt_sendDone(OpenQueueEntry_t* msg, owerror_t error) {      
      openqueue_freePacketBuffer(msg);
}

uint8_t openmqtt_sendPackage(uint8_t *buff,uint16_t lenght){
      // Packete que se envia
      OpenQueueEntry_t* outgoingPacket;

      outgoingPacket = openqueue_getFreePacketBuffer(COMPONENT_OPENMQTT);
      if (outgoingPacket==NULL) {
	    openserial_printError(
		  COMPONENT_OPENMQTT,
		  ERR_NO_FREE_PACKET_BUFFER,
		  (errorparameter_t)0,
		  (errorparameter_t)0
	    );
	    return FAILURE;
      }
      
      outgoingPacket->owner                         = COMPONENT_OPENMQTT;
      outgoingPacket->creator                       = COMPONENT_OPENMQTT;
      outgoingPacket->l4_protocol                   = IANA_UDP;
      outgoingPacket->l4_destination_port           = WKP_UDP_MQTT;
      outgoingPacket->l4_sourcePortORicmpv6Type     = WKP_UDP_MQTT;
      outgoingPacket->l3_destinationAdd.type        = ADDR_128B;
      memcpy(&outgoingPacket->l3_destinationAdd.addr_128b[0],openmqtt_dst_addr_connect,16);

      // add payload
      packetfunctions_reserveHeaderSize(outgoingPacket,lenght);
      memcpy(&outgoingPacket->payload[0],buff,lenght);

      // Paquete enviado
      if ((openudp_send(outgoingPacket))==E_FAIL) {
	    openqueue_freePacketBuffer(outgoingPacket);
	    return FAILURE;
      }
      
      return SUCCESS;
}

uint8_t openmqtt_connect(uint8_t *openmqtt_dst_addr, uint8_t id,
			 MQTTSNString *Topic_ini, MQTTSNString *Msg_ini){

      uint8_t sendbuf[MAX_PACKET_SIZE];
      int len = 0;
      
      if(ieee154e_isSynch() == FALSE){
	    return NO_RADIO; 
      }

      if (idmanager_getIsDAGroot()){
	    return IS_DAGROOT;
      }

      if (isConnected){
	    return IS_CONNECTED;
      }
      
      memcpy(&openmqtt_dst_addr_connect[0],openmqtt_dst_addr,16);
      memcpy(&Topic,Topic_ini,sizeof(MQTTSNString));
      memcpy(&Msg,Msg_ini,sizeof(MQTTSNString));
      
      MQTTSNPacket_connectData options = MQTTSNPacket_connectData_initializer;
      char cadena[50];
      sprintf(cadena,"d:mqttsn:openwsn:%d", id);
      options.clientID.cstring = cadena;
      len = MQTTSNSerialize_connect(sendbuf, MAX_PACKET_SIZE, &options);
      if(openmqtt_sendPackage(&sendbuf[0], len) == FAILURE)
	    return FAILURE;
      return SUCCESS;
}

uint8_t openmqtt_publish(MQTTSN_topicid *topic,
			 uint8_t *payload, uint16_t length){
      int len = 0;
      uint8_t sendbuf[MAX_PACKET_SIZE];
      if(!isConnected)
	    return IS_CONNECTED;
      memcpy(&topic_snd,topic,sizeof(MQTTSN_topicid));
      if(puback_recv){
	    len = MQTTSNSerialize_publish(sendbuf, MAX_PACKET_SIZE, 0, 0, 0, 1,
					  *topic, (unsigned char*)payload, (int)length);
	    if(openmqtt_sendPackage(sendbuf, len) == FAILURE)                                          
		  return FAILURE;
	    puback_recv = FALSE;
      } else {
	    return PUBACK;
      }
      return SUCCESS;
}

bool openmqtt_isConnected(void){
      return isConnected;
}

uint8_t openmqtt_disconnect(void){
      uint8_t sendbuf[MAX_PACKET_SIZE];
      int len = 0;
      len = MQTTSNSerialize_disconnect(sendbuf, MAX_PACKET_SIZE, FAILURE);
      if(openmqtt_sendPackage(&sendbuf[0], len) == FAILURE)
	    return FAILURE;
      isConnected = FALSE;
      return SUCCESS;
}
